/*
 * $Id: backlink.C,v 1.17 1995/05/26 16:06:39 asf Exp $
 */

/*
    This file is part of Addventure
    Copyright (C) 1994, 1995  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <ctype.h>

#include <htmllist.H>

#include "backlink.H"
#include "addvutil.H"

/***************************************************************************/

int backlink_to_room( char *backlink ){
  /*
   * Extract just the room number from inside the parenthesis.
   * Return 0 if the room number could not be extracted
   */

  char roomstr[10];
  char *title_index;
  int co = 0;
  int ret;

  strcpy( roomstr, "" );

  title_index = strstr( backlink, "]" );
  if( title_index ){
    // We found the room
    // Move it to where the number starts
    while( isdigit( *(--title_index) ) );
    title_index++;
    // Copy each number till the closing brace
    while( (*title_index) != ']' ){
      roomstr[co] = *title_index;
      co++;
      title_index++;
    }
    // Terminate the string
    roomstr[co] = '\0';
    ret = atoi( roomstr );
  } else {
    // Something is wrong
    ret = 0;
  }

  return ret;
}

/***************************************************************************/

BackLinkNode::BackLinkNode(char *title, int sel,
                           BackLinkNode *prev, BackLinkNode *next)
:HtmlSNode( title, sel, prev, next ){
}

//---------------------------------------------------------------------------

BackLinkNode::~BackLinkNode(){
}

//---------------------------------------------------------------------------

char *BackLinkNode::title(){
  /*
   * Just calls the HtmlNode value()
   */
  return value();
}

//---------------------------------------------------------------------------

int BackLinkNode::room(){
  /*
   * Extract just the room number
   */

  return backlink_to_room( title() );
}

/***************************************************************************/

BackLink::BackLink( char *path, char *roomword ){
  /*
   * Load the file
   */

  // Store the name and an empty list
  filename = make_fname( path, "backlink", "addv" );
  rooms = new HtmlSList;

  // Put in a "New room" first link that is selected
  // This link is malloc()ed and never free()d... but...
  char *newroomstr;
  newroomstr = (char *)malloc( 80 * sizeof(char) );
  sprintf( newroomstr, "New %s", roomword );
  BackLinkNode *newroom = new BackLinkNode( newroomstr, 1 );
  rooms->append( *newroom );
  
  // Load the file
  FILE *roomfile;
  char  title[160];
  int  done = 0;

  if( roomfile = fopen( filename, "r" ) ){
    // There is data to read
    while( !done ){
      // Reat the data.  Set "done" if we get EOF (fgets() returns NULL)
      done = !fgets( title, 160, roomfile );
      if( !done ){
        // Only append if we read something
        append_list( title );
      }
    }
  } else {
    // Something wen't wrong
  }
  
}

//---------------------------------------------------------------------------

BackLink::~BackLink(){
  free( filename );
  delete rooms;
}

//---------------------------------------------------------------------------
// Accessors

HtmlSList *BackLink::roomlist(){
  return rooms;
}

//---------------------------------------------------------------------------
// Commands

// This is used by the sorting
int compare( BacklinkSort kind, BackLinkNode *left, BackLinkNode *right ){
  /*
   * Return <0 if left < right
   * Return 0  if left == right
   * Return >0 if left > right
   */
  int ret;

  switch( kind ){
  case SORT_TITLE:
    ret = strcmp( left->title(), right->title() );
    break;
  case SORT_ROOM:
    ret = left->room() - right->room();
    break;
  }

  return ret;
}


void BackLink::sort( BacklinkSort kind ){
  /*
   * Do the actual sorting
   */

  // rooms is the list

  BackLinkNode *newroom;        // The first node shouldn't be sorted

  BackLinkNode *last_sorted;
  BackLinkNode *last_unsorted;
  BackLinkNode *to_sort;
  BackLinkNode *comp;

  // Remove the first node temporarily
  rooms->first();
  newroom = (BackLinkNode *)(rooms->remove());

  last_sorted = (BackLinkNode *)(rooms->first());

  // Read and move to the end of the list, and compare to the last one sorted
  while( last_sorted != (last_unsorted = (BackLinkNode *)rooms->last()) ){

    // get the node to sort.  This will be the last one, and we should
    // be pointing to it.
    to_sort = (BackLinkNode *)(rooms->remove());

    // Find where to insert it
    comp = (BackLinkNode *)(rooms->first());
    while( (comp != last_sorted) &&
           (compare( kind, to_sort, comp ) > 0 ) ){
      // While this isn't the last item, and the value to be sorted
      // is greater than the value compared against, we should advance
      comp = (BackLinkNode *)(rooms->next());
    }

    // We're either at the correct point, or at the end.  Put it here.
    if( compare( kind, to_sort, comp ) <= 0 ){
      // It is less than or equal to this node, so insert before
      rooms->insert_before( *to_sort );
    } else {
      // We must be at the end of the sorted list, so append it
      rooms->insert_after( *to_sort );
      last_sorted = to_sort;
    } // end if
  } // end while

  // Return the first room
  rooms->first();
  rooms->insert_before( *newroom );
}

//---------------------------------------------------------------------------

void BackLink::sort_name(){
  /*
   * Sort the list bythe full text of the room title.
   */

  sort( SORT_TITLE );
}

//---------------------------------------------------------------------------

void BackLink::sort_number(){
  /*
   * Sort the list by room number
   */

  sort( SORT_ROOM );
}

//---------------------------------------------------------------------------
  
int BackLink::room_exists( int room ){
  /*
   * Search for the room number in the list of rooms.  Return 1 if
   * the room exists, 0 if it does not.
   */
  int ret = 0;
  BackLinkNode *n;

  n = (BackLinkNode *)rooms->first();
  // Loop while there is still a node and we havent found it
  while( n && !ret ){
    // ret will equal 1 if we found the room
    ret = ( n->room() == room );
    n = (BackLinkNode *)rooms->next();
  }

  return ret;
}

//---------------------------------------------------------------------------
// Operations

void BackLink::append_list( char *title ){
  /*
   * Append just to the list
   */

  char *newtitle;      // A copy of the name
  BackLinkNode *node;  // The new node to be appended

  // Make a copy of the title
  newtitle = (char *)malloc( null_strlen(title) + 1 );
  strcpy( newtitle, title );

  // Make the new node with the title.  None of those appended will be
  // initially selected.
  node = new BackLinkNode( newtitle );

  // Append to the list
  rooms->append( *node );

}

//---------------------------------------------------------------------------

void BackLink::append( char *title ){
  /*
   * Append the string both to the end of the list, and to the file.
   */

  FILE *file;

  append_list( title );

  // Append to the file
  set_permissions();
  if( file = fopen( filename, "a" ) ){
    // File opened ok
    fprintf( file, "%s\n", title );
    fclose( file );
  } else {
    // Something went wrong
    fprintf( stderr, "backlink: '%s' NOT open for append\n", filename );
  }
}
  
/***************************************************************************/

/*
 * $Log: backlink.C,v $
 * Revision 1.17  1995/05/26  16:06:39  asf
 * Fixed some compile problems with a couple of functions.
 *
 * Revision 1.16  1995/01/02  09:35:41  asf
 * Changes in backlink_to_room() to locate the room number now
 * that the "room word" can be anything.
 *
 * Revision 1.15  1995/01/01  23:41:19  asf
 * Do a malloc() in the constructor for what to call the line where
 * a new "room" is created.  This string is never freed, but is necessary
 * because its used in place elsewhere.
 *
 * Revision 1.14  1995/01/01  16:34:28  asf
 * Added support for the "room word"
 *
 * Revision 1.13  1994/12/19  20:43:14  asf
 * Who deleted those parenthesis?
 *
 * Revision 1.12  1994/08/24  18:54:34  asf
 * Fixed a bug with compare() that was comparing numbers backwards.
 *
 * Revision 1.11  1994/08/24  18:45:35  asf
 * Created a new sort() function and some support types and routines that
 * sort_name() and sort_number() will use.
 *
 * Revision 1.10  1994/08/24  18:12:25  asf
 * New Room is now placed at the top of the sorted list.
 *
 * Revision 1.9  1994/08/24  14:42:28  asf
 * Fixed sort by name.  Involved changes to the List routines.
 * (Still need to make sure the "New Room" item does not get sorted in.)
 *
 * Revision 1.8  1994/08/23  20:46:24  asf
 * Wrote sort_name()
 *
 * Revision 1.7  1994/08/15  19:33:29  asf
 * Converted calls from strlen() to null_strlen()
 *
 * Revision 1.6  1994/08/14  07:57:40  asf
 * Removed some debugging statements
 *
 * Revision 1.5  1994/08/14  01:12:47  asf
 * Added some debugging.
 * Split append() and append_list().
 * Cleaned up the file appending.
 *
 * Revision 1.4  1994/08/13  19:40:26  asf
 * Thats "fprintf", not "printf" dummy.
 *
 * Revision 1.3  1994/08/13  19:28:10  asf
 * Append actually appends to the file as well now.  How radical.
 *
 * Revision 1.2  1994/08/13  19:14:33  asf
 * Corrected many bugs.
 * Made the function to convert a string to the room number a seperately
 * callable function otuside the object.
 *
 * Revision 1.1  1994/08/08  21:00:32  asf
 * Initial revision
 *
 */
