/*
 * $Id: error.C,v 1.9 1995/10/05 21:09:24 asf Exp $
 */

/*
    This file is part of Addventure
    Copyright (C) 1994  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <htmlfile.H>

#include "addvutil.H"
#include "error.H"

/***************************************************************************/

void addventure_error( char *module, AddvErrorType kind,
                       char *roomw, char *desc ){

  int backup = 0;    // Should we tell the person to try again?
  HtmlFile f;

  char *roomword;
  if( roomw ){
    roomword = null_strdup( roomw );
  } else {
    roomword = null_strdup( "room" );
  }

  f.content_type();

  f.title( "Addventure error" );
  f.header( 1, "Addventure error" );

  switch( kind ){
  case ADDVERR_UNKNOWN:
    f.puts( "An error has occurred. " );
    f.puts( "Your reply was not valid for some reason." );
    backup = 1;
    break;
  case ADDVERR_LOCKED:
    // This should be handled by the program
    break;
  case ADDVERR_INVALID_TARGET:
    f.puts( "One or more of the targets you requested is not a legal target ");
    f.puts( roomword );
    f.puts( "."  );
    backup = 1;
    break;
  case ADDVERR_NOT_EXIST:
    f.puts( "The " );
    f.puts( roomword );
    f.puts( " you are trying to edit was not created." );
    break;
  case ADDVERR_EMPTY:
    f.puts( "You either did not enter a description, or tried to create a " );
    f.puts( roomword );
    f.puts( " with zero options." );
    backup = 1;
    break;
  case ADDVERR_HTML:
    char *reason[4];
    f.puts( "The following HTML codes are not allowed:" );
    reason[0] = "Anchors, &lt;a&gt; and &lt;/a&gt;, in option descriptions.";
    reason[1] = "Preformatting, &lt;pre&gt; and &lt;/pre&gt;, in either"
      "the text or option descriptions.";
    reason[2] = "Comments in either the text or option descriptions.";
    reason[3] = NULL;
    f.unnumlist( reason );
    backup = 1;
    break;
  case ADDVERR_EXISTS:
    f.puts( "Somehow, that " );
    f.puts( roomword );
    f.puts( " was already being created." );
    break;
  case ADDVERR_FINISHED:
    f.puts( "The " );
    f.puts( roomword );
    f.puts( " has already been created.  It is too late to remove it." );
    break;
  case ADDVERR_NONEXTROOM:
    f.puts( "For some reason, Addventure could not allocate a new " );
    f.puts( roomword );
    f.puts( " for your choices.  Your " );
    f.puts( roomword );
    f.puts( " was not created." );
    break;
  }
  f.para();

  if( backup ){
    f.puts( "Please go back a screen and try again.\n" );
  }

  if( desc ){
    f.raw( desc );
    f.para();
  }

  f.line();
  f.puts( "Problem occurred in: <i>" );
  f.puts( module );
  f.puts( "</i>" );
  f.para();

  free( roomword );
}

/***************************************************************************/

/*
 * $Log: error.C,v $
 * Revision 1.9  1995/10/05  21:09:24  asf
 * Added the No Next Room error
 *
 * Revision 1.8  1995/05/26  16:02:22  asf
 * Fixed a compile problem with addventure_error().
 *
 * Revision 1.7  1995/01/28  17:00:25  asf
 * Added more types of bad HTML that is reported.
 *
 * Revision 1.6  1995/01/01  23:01:05  asf
 * Fixed some spacing.
 *
 * Revision 1.5  1995/01/01  17:37:36  asf
 * Added the "room word" to the error messages
 *
 * Revision 1.4  1994/12/25  11:07:38  asf
 * Added ADDVERR_HTML
 *
 * Revision 1.3  1994/12/17  19:11:25  asf
 * Changed the message for ADDVERR_EMPTY slightly (which really makes the name
 * wrong.. but...)
 *
 * Revision 1.2  1994/09/12  12:00:50  asf
 * Moved the line that reported the module to a less obvious place.
 *
 * Revision 1.1  1994/08/26  09:08:52  asf
 * Initial revision
 *
 */
