/*
 * $Id: response.C,v 1.13 1996/01/27 22:09:43 asf Exp $
 */

/*
    This file is part of Addventure
    Copyright (C) 1994, 1995, 1996  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>

#include "response.H"
#include "option.H"
#include "addvutil.H"
#include "setaddv.H"

#define RESPONSE_SEP "\t"

/***************************************************************************/

Response::Response( char *string ){
  /*
   * parse it
   */

  char *temp_string = null_strdup( string );

  // Break up the string
  char *parent_str  = strtok( temp_string, RESPONSE_SEP );
  char *room_str    = strtok( NULL, RESPONSE_SEP );
  char *address_str = strtok( NULL, RESPONSE_SEP );

  // Store it
  _parent = atoi( parent_str );
  _room = atoi( room_str );
  _address = null_strdup( address_str );

  free( temp_string );
}

//---------------------------------------------------------------------------

Response::Response( int parent, int room, char *address ){

  _parent = parent;
  _room = room;
  _address = null_strdup( address );
  
}

//---------------------------------------------------------------------------

Response::~Response(){
  
  free( _address );
  
}

//---------------------------------------------------------------------------
// Accessors

int Response::parent(){
  return _parent;
}

//---------------------------------------------------------------------------

int Response::room(){

  return _room;
  
}

//---------------------------------------------------------------------------

char *Response::address(){

  return _address;
  
}

//---------------------------------------------------------------------------

char *Response::string(){
  // The single line representation

  static char tmp_str[1000];

  strcpy( tmp_str, "" );
  sprintf( tmp_str, "%d%s%d%s%s", _parent, RESPONSE_SEP,
           _room, RESPONSE_SEP, _address );

  return tmp_str;
}

/***************************************************************************/

ResponseNode::ResponseNode( char *string ):
  Response( string ){
  //parse it
}

//---------------------------------------------------------------------------

ResponseNode::ResponseNode( int parent, int room, char *address ):
  Response( parent, room, address ){
}

//---------------------------------------------------------------------------

ResponseNode::~ResponseNode(){
}

/***************************************************************************/

ResponseFile::ResponseFile( char *filepath, int load  ){

  // It is never loaded initially, the call to load() can/will change this
  _loaded = 0;
  
  // Create the storage
  _responselist = new List;

  // Copy just the path
  _filepath = null_strdup( filepath );
  
  // Get the name
  _filename = make_fname( filepath, "response", "addv" );

  // Possibly load the file
  if( load ){
    this->load();
  }
}

//---------------------------------------------------------------------------

ResponseFile::~ResponseFile(){

  free( _filepath );
  free( _filename );
  delete _responselist;
  
}

//---------------------------------------------------------------------------
// Accessors

List *ResponseFile::responselist(){

  return _responselist;
  
}

//---------------------------------------------------------------------------
// File actions

void ResponseFile::load(){

  FILE *f;

  char buffer[1000];
  char *response;

  ResponseNode *node;

  if( !_loaded ){
    _loaded = 1;

    if( f = fopen( _filename, "r" ) ){
      do {
        response = fgets( buffer, 1000, f );
        if( response ){
          node = new ResponseNode( buffer );
          _responselist->append( *node );
        }
      } while( response );
      fclose( f );
    }
  }
  
}

//---------------------------------------------------------------------------

void ResponseFile::add( int parent, OptionArray &children, char *author ){

  int co;
  int numopt = children.size();

  Option *option;

  for( co=1; co<=numopt; co++ ){
    option = children[co];

    add( parent, option->room(), author );
  }
}

//---------------------------------------------------------------------------

void ResponseFile::add( int parent, int child, char *author ){

  FILE *f;

  // Create a node with the data
  ResponseNode *node = new ResponseNode( parent, child, author );
    
  // Append to the list if the rest of the file is already loaded
  if( _loaded ){
    _responselist->append( *node );
  }

  // Regardless, put it at the end of the file
  set_permissions();
  if( f = fopen( _filename, "a" ) ){
    fprintf( f, "%s\n", node->string() );
    fclose( f );
  }

}

//---------------------------------------------------------------------------

int ResponseFile::respond( int room ){
  /*
   * If there is an author for this room, respond
   */

  int found = 0;    // Have we found the room
  ResponseNode *node;

  int parent;

  // We need to have the file loaded
  if( !_loaded ){
    load();
  }

  // Locate the room
  node = (ResponseNode *)_responselist->first();
  while( node && (!found) ){
    if( node->room() == room ){
      found = 1;
      parent = node->parent();
    } else {
      node = (ResponseNode *)_responselist->next();
    }
  }

  // Send email if there was a registration
  if( found ){
    FILE *program;
    char command_line[255];

    // Get the title and the room word, if we can
    char *title = NULL;
    char *roomword;
    if( _filepath ){
      SetAddv setting( _filepath );
      title = setting.game_title();
      roomword = setting.roomword();
    } else {
      roomword = "room";
    }

#ifndef SENDMAIL    
    sprintf( command_line,
             "%s -s 'Addventure Extension' %s",
             MAILPROGRAM,
             node->address()
           );
#else
    sprintf( command_line,
             "%s -f%s -t",
             MAILPROGRAM,
             MAILFROM
           );
#endif    

    program = popen( command_line, "w" );

#ifdef SENDMAIL
    fprintf( program, "To: %s\n", node->address() );
    fprintf( program, "From: %s\n", MAILFROM );
    fprintf( program, "Subject: Addventure Extension\n\n" );
#endif
    
    if( title ){
      fprintf( program, "In the game '%s'\n", title );
    }
    
    fprintf( program, "%s %d, a child of %s %d, has been created.\n\n",
             roomword, room, roomword, parent );
    fprintf( program, "This message was automatically generated.\n" );
    fprintf( program, "%c\n\n", 0x04 );   // C-d to end the file entry

    pclose( program );
  }
  
}

/***************************************************************************/

/*
 * $Log: response.C,v $
 * Revision 1.13  1996/01/27  22:09:43  asf
 * Added components for sendmail support
 *
 * Revision 1.12  1995/05/26  16:10:42  asf
 * Changes to add() to fix compile problems.
 *
 * Revision 1.11  1995/01/02  10:38:39  asf
 * Fixed some parameter typos in response()
 *
 * Revision 1.10  1995/01/01  16:43:02  asf
 * Added support for the "room word"
 *
 * Revision 1.9  1994/12/15  20:31:26  asf
 * Accidentally swapped strings to parse in the Response line parser.  Fixed.
 *
 * Revision 1.8  1994/12/15  19:50:49  asf
 * Added the capability to reply with the parent of the room.
 * Added the capability to reply with the title of the game, if it exists.
 *
 * Revision 1.7  1994/12/13  22:22:47  asf
 * (Hopefully) fixed mailing bug by using define of MAILPROGRAM
 * instead of hardcoding the program name in.
 *
 * Revision 1.6  1994/11/06  18:09:07  asf
 * Changed the mail sending to using popen().
 *
 * Revision 1.5  1994/11/06  16:07:12  asf
 * Changed quoting on subject line
 *
 * Revision 1.4  1994/11/06  15:58:46  asf
 * Removed newline from mail command
 *
 * Revision 1.3  1994/11/06  15:19:29  asf
 * OptionArray is 1 based!
 * Changed the filename from "stat" to "response" as it should be.
 *
 * Revision 1.2  1994/11/06  15:17:17  asf
 * Response::string() needed a return statement
 *
 * Revision 1.1  1994/11/06  13:05:21  asf
 * Initial revision
 *
 */
